# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'giusas/gui/ui/wgiusas.ui'
#
# Created by: PyQt5 UI code generator 5.9
#
# WARNING! All changes made in this file will be lost!

from PyQt5 import QtCore, QtGui, QtWidgets

class Ui_WGiuSAS(object):
    def setupUi(self, WGiuSAS):
        WGiuSAS.setObjectName("WGiuSAS")
        WGiuSAS.resize(953, 700)
        WGiuSAS.setLocale(QtCore.QLocale(QtCore.QLocale.English, QtCore.QLocale.UnitedStates))
        WGiuSAS.setTabShape(QtWidgets.QTabWidget.Rounded)
        self.centralwidget = QtWidgets.QWidget(WGiuSAS)
        self.centralwidget.setObjectName("centralwidget")
        self.horizontalLayout = QtWidgets.QHBoxLayout(self.centralwidget)
        self.horizontalLayout.setObjectName("horizontalLayout")
        WGiuSAS.setCentralWidget(self.centralwidget)
        self.menubar = QtWidgets.QMenuBar(WGiuSAS)
        self.menubar.setGeometry(QtCore.QRect(0, 0, 953, 49))
        self.menubar.setObjectName("menubar")
        self.menuFile = QtWidgets.QMenu(self.menubar)
        self.menuFile.setObjectName("menuFile")
        self.menuView = QtWidgets.QMenu(self.menubar)
        self.menuView.setObjectName("menuView")
        WGiuSAS.setMenuBar(self.menubar)
        self.statusbar = QtWidgets.QStatusBar(WGiuSAS)
        self.statusbar.setObjectName("statusbar")
        WGiuSAS.setStatusBar(self.statusbar)
        self.dockFiles = QtWidgets.QDockWidget(WGiuSAS)
        self.dockFiles.setObjectName("dockFiles")
        self.dockWidgetContents = QtWidgets.QWidget()
        self.dockWidgetContents.setObjectName("dockWidgetContents")
        self.dockFiles.setWidget(self.dockWidgetContents)
        WGiuSAS.addDockWidget(QtCore.Qt.DockWidgetArea(8), self.dockFiles)
        self.dockParams = QtWidgets.QDockWidget(WGiuSAS)
        self.dockParams.setObjectName("dockParams")
        self.dockWidgetContents_2 = QtWidgets.QWidget()
        self.dockWidgetContents_2.setObjectName("dockWidgetContents_2")
        self.dockParams.setWidget(self.dockWidgetContents_2)
        WGiuSAS.addDockWidget(QtCore.Qt.DockWidgetArea(1), self.dockParams)
        self.actionRotateImage = QtWidgets.QAction(WGiuSAS)
        icon = QtGui.QIcon()
        icon.addPixmap(QtGui.QPixmap(":/rotate"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.actionRotateImage.setIcon(icon)
        self.actionRotateImage.setObjectName("actionRotateImage")
        self.actionFlipImageH = QtWidgets.QAction(WGiuSAS)
        icon1 = QtGui.QIcon()
        icon1.addPixmap(QtGui.QPixmap(":/flip-horizontal"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.actionFlipImageH.setIcon(icon1)
        self.actionFlipImageH.setObjectName("actionFlipImageH")
        self.actionFlipImageV = QtWidgets.QAction(WGiuSAS)
        icon2 = QtGui.QIcon()
        icon2.addPixmap(QtGui.QPixmap(":/flip-vertical"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.actionFlipImageV.setIcon(icon2)
        self.actionFlipImageV.setObjectName("actionFlipImageV")
        self.actionAdjustLevels = QtWidgets.QAction(WGiuSAS)
        icon3 = QtGui.QIcon()
        icon3.addPixmap(QtGui.QPixmap(":/adjust"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.actionAdjustLevels.setIcon(icon3)
        self.actionAdjustLevels.setObjectName("actionAdjustLevels")
        self.actionLogarithmicScale = QtWidgets.QAction(WGiuSAS)
        self.actionLogarithmicScale.setCheckable(True)
        self.actionLogarithmicScale.setObjectName("actionLogarithmicScale")
        self.actionQuit = QtWidgets.QAction(WGiuSAS)
        self.actionQuit.setObjectName("actionQuit")
        self.actionParameters = QtWidgets.QAction(WGiuSAS)
        self.actionParameters.setCheckable(True)
        self.actionParameters.setChecked(True)
        self.actionParameters.setObjectName("actionParameters")
        self.actionFiles = QtWidgets.QAction(WGiuSAS)
        self.actionFiles.setCheckable(True)
        self.actionFiles.setChecked(True)
        self.actionFiles.setObjectName("actionFiles")
        self.menuFile.addAction(self.actionQuit)
        self.menuView.addAction(self.actionParameters)
        self.menuView.addAction(self.actionFiles)
        self.menubar.addAction(self.menuFile.menuAction())
        self.menubar.addAction(self.menuView.menuAction())

        self.retranslateUi(WGiuSAS)
        QtCore.QMetaObject.connectSlotsByName(WGiuSAS)

    def retranslateUi(self, WGiuSAS):
        _translate = QtCore.QCoreApplication.translate
        WGiuSAS.setWindowTitle(_translate("WGiuSAS", "GiuSAS - GISAXS and GIWAXS processing application"))
        self.menuFile.setTitle(_translate("WGiuSAS", "&File"))
        self.menuView.setTitle(_translate("WGiuSAS", "&View"))
        self.actionRotateImage.setText(_translate("WGiuSAS", "Rotate image clockwise"))
        self.actionRotateImage.setToolTip(_translate("WGiuSAS", "Rotate image clockwise"))
        self.actionRotateImage.setShortcut(_translate("WGiuSAS", "Ctrl+R"))
        self.actionFlipImageH.setText(_translate("WGiuSAS", "Flip image horizontally"))
        self.actionFlipImageH.setToolTip(_translate("WGiuSAS", "Flip image horizontally"))
        self.actionFlipImageH.setShortcut(_translate("WGiuSAS", "Ctrl+H"))
        self.actionFlipImageV.setText(_translate("WGiuSAS", "Flip image vertically"))
        self.actionFlipImageV.setToolTip(_translate("WGiuSAS", "Flip image vertically"))
        self.actionFlipImageV.setShortcut(_translate("WGiuSAS", "Ctrl+V"))
        self.actionAdjustLevels.setText(_translate("WGiuSAS", "Adjust levels"))
        self.actionAdjustLevels.setToolTip(_translate("WGiuSAS", "Adjust image levels"))
        self.actionAdjustLevels.setShortcut(_translate("WGiuSAS", "Ctrl+A"))
        self.actionLogarithmicScale.setText(_translate("WGiuSAS", "Log"))
        self.actionLogarithmicScale.setShortcut(_translate("WGiuSAS", "Ctrl+L"))
        self.actionQuit.setText(_translate("WGiuSAS", "Quit"))
        self.actionParameters.setText(_translate("WGiuSAS", "Parameters"))
        self.actionFiles.setText(_translate("WGiuSAS", "Files"))

from . import resources_rc
